unit tcSpecificFunctions;

interface

uses
  SysConst, Windows, SysUtils, Classes, IniFiles, ShellAPI, VCL.Graphics;

function ExpandEnvVar(const StringToExpand, CurrentDIRECTORY: string; GetUNC: Boolean = False): string;

function GetTCINIWithRedirectedSection(const Section: string; CheckFileExists: Boolean = True): TINIFile;
function GetInternalTCINISection(const Section: string): TStringList;
function GetInternalTCINISectionEx(const Section: string): THashedStringList;

function LoadIconWithScaleDown(hinst: HINST; pszName: LPCWSTR; cx: Integer;
  cy: Integer; var phico: HICON): HResult; stdcall;

implementation

function LoadIconWithScaleDown; external 'comctl32.dll' name 'LoadIconWithScaleDown' delayed;

function ExpandEnvVar(const StringToExpand, CurrentDIRECTORY: string; GetUNC: Boolean = False): string;
var
  Path, CurrentDir: string;
begin
  Result := '';
  if StringToExpand = '' then
    Exit;
  SetLength(Path, 3200);
  ExpandEnvironmentStrings(PChar(StringToExpand), PChar(Path), Length(Path) - 1);
  SetLength(Path, StrLen(PChar(Path)));
  if CurrentDIRECTORY <> '' then
  begin
    CurrentDir := GetCurrentDir;
    SetCurrentDir(CurrentDIRECTORY);
  end;
  if GetUNC then
    Result := ExpandUNCFileName(Path)
  else
    Result := ExpandFileName(Path);
  if CurrentDIRECTORY <> '' then
    SetCurrentDir(CurrentDir);
end;

function GetTCINIWithRedirectedSection(const Section: string; CheckFileExists: Boolean): TINIFile;
var
  TCIni, RedirectedSection: string;
begin
  Result := nil;
  TCIni := GetEnvironmentVariable('COMMANDER_INI');
  if TCIni <> '' then
  try
    if FileExists(TCIni) then
    begin
      Result := TINIFile.Create(TCIni);
      RedirectedSection := Result.ReadString(Section, 'RedirectSection', '');
      if RedirectedSection <> '' then
        if RedirectedSection <> '0' then
        begin
          if RedirectedSection = '1' then
            RedirectedSection := Result.ReadString('Configuration',
              'AlternateUserIni', '');
          //SetCurrentDir(TCIni);
          //RedirectedSection := ExpandEnvVar(RedirectedSection, ExtractFileDir(GetModuleName(MainInstance)));
          RedirectedSection := ExpandEnvVar(RedirectedSection, ExtractFileDir(TCIni));
          if CheckFileExists and FileExists(RedirectedSection) then
          begin
            FreeAndNil(Result);
            Result := TINIFile.Create(RedirectedSection);
          end;
        end;
    end;
  except
    // do nothing
  end;
end;

function GetInternalTCINISection(const Section: string): TStringList;
var
  INI: TINIFile;
begin
  Result := TStringList.Create;
  begin
    try
      INI := GetTCINIWithRedirectedSection(Section);
      if not Assigned(INI) then
        Exit;
      try
        INI.ReadSectionValues(Section, Result);
      finally
        INI.Free;
      end;
    except
      // do nothing
    end;
  end;
end;

function GetInternalTCINISectionEx(const Section: string): THashedStringList;
var
  INI: TINIFile;
begin
  Result := THashedStringList.Create;
  begin
    try
      INI := GetTCINIWithRedirectedSection(Section);
      if not Assigned(INI) then
        Exit;
      try
        INI.ReadSectionValues(Section, Result);
      finally
        INI.Free;
      end;
    except
      // do nothing
    end;
  end;
end;

end.
